@@ -1,9 +1,8 @@
-/* $Id: chars.c 4534 2011-02-24 02:47:25Z astyanax $ */
 /**************************************************************************
  *   chars.c                                                              *
  *                                                                        *
- *   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009   *
- *   Free Software Foundation, Inc.                                       *
+ *   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,  *
+ *   2010, 2011, 2013, 2014 Free Software Foundation, Inc.                *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 3, or (at your option)  *
@@ -36,12 +35,6 @@
 
 static bool use_utf8 = FALSE;
 	/* Whether we've enabled UTF-8 support. */
-static const wchar_t bad_wchar = 0xFFFD;
-	/* If we get an invalid multibyte sequence, we treat it as
-	 * Unicode FFFD (Replacement Character), unless we're searching
-	 * for a match to it. */
-static const char *const bad_mbchar = "\xEF\xBF\xBD";
-static const int bad_mbchar_len = 3;
 
 /* Enable UTF-8 support. */
 void utf8_init(void)
@@ -54,7 +47,19 @@ bool using_utf8(void)
 {
     return use_utf8;
 }
-#endif
+#endif /* ENABLE_UTF8 */
+
+/* Concatenate two allocated strings, and free the second. */
+char *addstrings(char* str1, size_t len1, char* str2, size_t len2)
+{
+    str1 = charealloc(str1, len1 + len2 + 1);
+    str1[len1] = '\0';
+
+    strncat(&str1[len1], str2, len2);
+    free(str2);
+
+    return str1;
+}
 
 #ifndef HAVE_ISBLANK
 /* This function is equivalent to isblank(). */
@@ -72,19 +77,18 @@ bool niswblank(wchar_t wc)
 }
 #endif
 
-/* Return TRUE if the value of c is in byte range, and FALSE
- * otherwise. */
+/* Return TRUE if the value of c is in byte range, and FALSE otherwise. */
 bool is_byte(int c)
 {
     return ((unsigned int)c == (unsigned char)c);
 }
 
-static void mbtowc_reset(void)
+void mbtowc_reset(void)
 {
     IGNORE_CALL_RESULT(mbtowc(NULL, NULL, 0));
 }
 
-static void wctomb_reset(void)
+void wctomb_reset(void)
 {
     IGNORE_CALL_RESULT(wctomb(NULL, 0));
 }
@@ -100,7 +104,7 @@ bool is_alnum_mbchar(const char *c)
 
 	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
 	    mbtowc_reset();
-	    wc = bad_wchar;
+	    return 0;
 	}
 
 	return iswalnum(wc);
@@ -120,7 +124,7 @@ bool is_blank_mbchar(const char *c)
 
 	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
 	    mbtowc_reset();
-	    wc = bad_wchar;
+	    return 0;
 	}
 
 	return iswblank(wc);
@@ -140,20 +144,9 @@ bool is_ascii_cntrl_char(int c)
  * handles high-bit control characters. */
 bool is_cntrl_char(int c)
 {
-    return (-128 <= c && c < -96) || (0 <= c && c < 32) ||
-	(127 <= c && c < 160);
+    return ((c & 0x60) == 0 || c == 127);
 }
 
-#ifdef ENABLE_UTF8
-/* This function is equivalent to iscntrl() for wide characters, except
- * in that it also handles wide control characters with their high bits
- * set. */
-bool is_cntrl_wchar(wchar_t wc)
-{
-    return (0 <= wc && wc < 32) || (127 <= wc && wc < 160);
-}
-#endif
-
 /* This function is equivalent to iscntrl() for multibyte characters,
  * except in that it also handles multibyte control characters with
  * their high bits set. */
@@ -163,14 +156,8 @@ bool is_cntrl_mbchar(const char *c)
 
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	wchar_t wc;
-
-	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
-	    mbtowc_reset();
-	    wc = bad_wchar;
-	}
-
-	return is_cntrl_wchar(wc);
+	return ((c[0] & 0xE0) == 0 || c[0] == 127 ||
+		((signed char)c[0] == -62 && (signed char)c[1] < -96));
     } else
 #endif
 	return is_cntrl_char((unsigned char)*c);
@@ -184,11 +171,10 @@ bool is_punct_mbchar(const char *c)
 #ifdef ENABLE_UTF8
     if (use_utf8) {
 	wchar_t wc;
-	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
 
-	if (c_mb_len < 0) {
+	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
 	    mbtowc_reset();
-	    wc = bad_wchar;
+	    return 0;
 	}
 
 	return iswpunct(wc);
@@ -197,117 +183,100 @@ bool is_punct_mbchar(const char *c)
 	return ispunct((unsigned char)*c);
 }
 
-/* Return TRUE for a multibyte character found in a word (currently only
- * an alphanumeric or punctuation character, and only the latter if
- * allow_punct is TRUE) and FALSE otherwise. */
+/* Return TRUE when the given multibyte character c is a word-forming
+ * character (that is: alphanumeric, or specified in wordchars, or
+ * punctuation when allow_punct is TRUE), and FALSE otherwise. */
 bool is_word_mbchar(const char *c, bool allow_punct)
 {
     assert(c != NULL);
 
-    return is_alnum_mbchar(c) || (allow_punct ? is_punct_mbchar(c) :
-	FALSE);
+    if (*c == '\0')
+	return FALSE;
+
+    if (is_alnum_mbchar(c))
+	return TRUE;
+
+    if (word_chars != NULL && *word_chars != '\0') {
+	bool wordforming;
+	char *symbol = charalloc(MB_CUR_MAX + 1);
+	int symlen = parse_mbchar(c, symbol, NULL);
+
+	symbol[symlen] = '\0';
+	wordforming = (strstr(word_chars, symbol) != NULL);
+	free(symbol);
+
+	return wordforming;
+    }
+
+    return (allow_punct && is_punct_mbchar(c));
 }
 
-/* c is a control character.  It displays as ^@, ^?, or ^[ch], where ch
- * is (c + 64).  We return that character. */
-char control_rep(char c)
+/* Return the visible representation of control character c. */
+char control_rep(const signed char c)
 {
     assert(is_cntrl_char(c));
 
-    /* Treat newlines embedded in a line as encoded nulls. */
+    /* An embedded newline is an encoded null. */
     if (c == '\n')
 	return '@';
     else if (c == NANO_CONTROL_8)
 	return '?';
+    else if (c == -97)
+	return '=';
+    else if (c < 0)
+	return c + 224;
     else
 	return c + 64;
 }
 
+/* Return the visible representation of multibyte control character c. */
+char control_mbrep(const char *c)
+{
+    assert(c != NULL);
+
 #ifdef ENABLE_UTF8
-/* c is a wide control character.  It displays as ^@, ^?, or ^[ch],
- * where ch is (c + 64).  We return that wide character. */
-wchar_t control_wrep(wchar_t wc)
-{
-    assert(is_cntrl_wchar(wc));
-
-    /* Treat newlines embedded in a line as encoded nulls. */
-    if (wc == '\n')
-	return '@';
-    else if (wc == NANO_CONTROL_8)
-	return '?';
-    else
-	return wc + 64;
-}
+    if (use_utf8) {
+	if (0 <= c[0] && c[0] <= 127)
+	    return control_rep(c[0]);
+	else
+	    return control_rep(c[1]);
+    } else
 #endif
+	return control_rep(*c);
+}
 
-/* c is a multibyte control character.  It displays as ^@, ^?, or ^[ch],
- * where ch is (c + 64).  We return that multibyte character.  If crep
- * is an invalid multibyte sequence, it will be replaced with Unicode
- * 0xFFFD (Replacement Character). */
-char *control_mbrep(const char *c, char *crep, int *crep_len)
+/* Assess how many bytes the given (multibyte) character occupies.  Return -1
+ * if the byte sequence is invalid, and return the number of bytes minus 8
+ * when it encodes an invalid codepoint.  Also, in the second parameter,
+ * return the number of columns that the character occupies. */
+int length_of_char(const char *c, int *width)
 {
-    assert(c != NULL && crep != NULL && crep_len != NULL);
+    assert(c != NULL);
 
 #ifdef ENABLE_UTF8
     if (use_utf8) {
 	wchar_t wc;
+	int charlen = mbtowc(&wc, c, MB_CUR_MAX);
 
-	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
+	/* If the sequence is invalid... */
+	if (charlen < 0) {
 	    mbtowc_reset();
-	    *crep_len = bad_mbchar_len;
-	    strncpy(crep, bad_mbchar, *crep_len);
-	} else {
-	    *crep_len = wctomb(crep, control_wrep(wc));
-
-	    if (*crep_len < 0) {
-		wctomb_reset();
-		*crep_len = 0;
-	    }
+	    return -1;
 	}
-    } else {
-#endif
-	*crep_len = 1;
-	*crep = control_rep(*c);
-#ifdef ENABLE_UTF8
-    }
-#endif
 
-    return crep;
-}
-
-/* c is a multibyte non-control character.  We return that multibyte
- * character.  If crep is an invalid multibyte sequence, it will be
- * replaced with Unicode 0xFFFD (Replacement Character). */
-char *mbrep(const char *c, char *crep, int *crep_len)
-{
-    assert(c != NULL && crep != NULL && crep_len != NULL);
-
-#ifdef ENABLE_UTF8
-    if (use_utf8) {
-	wchar_t wc;
-
-	/* Reject invalid Unicode characters. */
-	if (mbtowc(&wc, c, MB_CUR_MAX) < 0 || !is_valid_unicode(wc)) {
-	    mbtowc_reset();
-	    *crep_len = bad_mbchar_len;
-	    strncpy(crep, bad_mbchar, *crep_len);
-	} else {
-	    *crep_len = wctomb(crep, wc);
-
-	    if (*crep_len < 0) {
-		wctomb_reset();
-		*crep_len = 0;
-	    }
+	/* If the codepoint is invalid... */
+	if (!is_valid_unicode(wc))
+	    return charlen - 8;
+	else {
+	    *width = wcwidth(wc);
+	    /* If the codepoint is unassigned, assume a width of one. */
+	    if (*width < 0)
+		*width = 1;
+	    return charlen;
 	}
-    } else {
+    } else
 #endif
-	*crep_len = 1;
-	*crep = *c;
-#ifdef ENABLE_UTF8
-    }
-#endif
-
-    return crep;
+	return 1;
 }
 
 /* This function is equivalent to wcwidth() for multibyte characters. */
@@ -322,15 +291,13 @@ int mbwidth(const char *c)
 
 	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
 	    mbtowc_reset();
-	    wc = bad_wchar;
+	    return 1;
 	}
 
 	width = wcwidth(wc);
 
-	if (width == -1) {
-	    wc = bad_wchar;
-	    width = wcwidth(wc);
-	}
+	if (width == -1)
+	    return 1;
 
 	return width;
     } else
@@ -369,13 +336,12 @@ char *make_mbchar(long chr, int *chr_mb_len)
 	    wctomb_reset();
 	    *chr_mb_len = 0;
 	}
-    } else {
+    } else
 #endif
+    {
 	*chr_mb_len = 1;
 	chr_mb = mallocstrncpy(NULL, (char *)&chr, 1);
-#ifdef ENABLE_UTF8
     }
-#endif
 
     return chr_mb;
 }
@@ -395,15 +361,14 @@ int parse_mbchar(const char *buf, char *chr, size_t *col)
 	/* Get the number of bytes in the multibyte character. */
 	buf_mb_len = mblen(buf, MB_CUR_MAX);
 
-	/* If buf contains an invalid multibyte character, only
-	 * interpret buf's first byte. */
+	/* When the multibyte sequence is invalid, only take the first byte. */
 	if (buf_mb_len < 0) {
 	    IGNORE_CALL_RESULT(mblen(NULL, 0));
 	    buf_mb_len = 1;
 	} else if (buf_mb_len == 0)
 	    buf_mb_len++;
 
-	/* Save the multibyte character in chr. */
+	/* When requested, store the multibyte character in chr. */
 	if (chr != NULL) {
 	    int i;
 
@@ -411,60 +376,46 @@ int parse_mbchar(const char *buf, char *chr, size_t *col)
 		chr[i] = buf[i];
 	}
 
-	/* Save the column width of the wide character in col. */
+	/* When requested, store the width of the wide character in col. */
 	if (col != NULL) {
 	    /* If we have a tab, get its width in columns using the
 	     * current value of col. */
 	    if (*buf == '\t')
 		*col += tabsize - *col % tabsize;
-	    /* If we have a control character, get its width using one
-	     * column for the "^" that will be displayed in front of it,
-	     * and the width in columns of its visible equivalent as
-	     * returned by control_mbrep(). */
+	    /* If we have a control character, it's two columns wide: one
+	     * column for the "^", and one for the visible character. */
 	    else if (is_cntrl_mbchar(buf)) {
-		char *ctrl_buf_mb = charalloc(MB_CUR_MAX);
-		int ctrl_buf_mb_len;
-
-		(*col)++;
-
-		ctrl_buf_mb = control_mbrep(buf, ctrl_buf_mb,
-			&ctrl_buf_mb_len);
-
-		*col += mbwidth(ctrl_buf_mb);
-
-		free(ctrl_buf_mb);
+		*col += 2;
 	    /* If we have a normal character, get its width in columns
 	     * normally. */
 	    } else
 		*col += mbwidth(buf);
 	}
-    } else {
+    } else
 #endif
-	/* Get the number of bytes in the byte character. */
+    {
+	/* A byte character is one byte long. */
 	buf_mb_len = 1;
 
-	/* Save the byte character in chr. */
+	/* When requested, store the byte character in chr. */
 	if (chr != NULL)
 	    *chr = *buf;
 
+	/* When requested, store the width of the wide character in col. */
 	if (col != NULL) {
 	    /* If we have a tab, get its width in columns using the
 	     * current value of col. */
 	    if (*buf == '\t')
 		*col += tabsize - *col % tabsize;
-	    /* If we have a control character, it's two columns wide:
-	     * one column for the "^" that will be displayed in front of
-	     * it, and one column for its visible equivalent as returned
-	     * by control_mbrep(). */
+	    /* If we have a control character, it's two columns wide: one
+	     * column for the "^", and one for the visible character. */
 	    else if (is_cntrl_char((unsigned char)*buf))
 		*col += 2;
 	    /* If we have a normal character, it's one column wide. */
 	    else
 		(*col)++;
 	}
-#ifdef ENABLE_UTF8
     }
-#endif
 
     return buf_mb_len;
 }
@@ -473,22 +424,24 @@ int parse_mbchar(const char *buf, char *chr, size_t *col)
  * before the one at pos. */
 size_t move_mbleft(const char *buf, size_t pos)
 {
-    size_t pos_prev = pos;
+    size_t before, char_len = 0;
 
     assert(buf != NULL && pos <= strlen(buf));
 
     /* There is no library function to move backward one multibyte
-     * character.  Here is the naive, O(pos) way to do it. */
-    while (TRUE) {
-	int buf_mb_len = parse_mbchar(buf + pos - pos_prev, NULL, NULL);
+     * character.  So we just start groping for one at the farthest
+     * possible point. */
+    if (mb_cur_max() > pos)
+	before = 0;
+    else
+	before = pos - mb_cur_max();
 
-	if (pos_prev <= buf_mb_len)
-	    break;
-
-	pos_prev -= buf_mb_len;
+    while (before < pos) {
+	char_len = parse_mbchar(buf + before, NULL, NULL);
+	before += char_len;
     }
 
-    return pos - pos_prev;
+    return before - char_len;
 }
 
 /* Return the index in buf of the beginning of the multibyte character
@@ -502,14 +455,14 @@ size_t move_mbright(const char *buf, size_t pos)
 /* This function is equivalent to strcasecmp(). */
 int nstrcasecmp(const char *s1, const char *s2)
 {
-    return strncasecmp(s1, s2, (size_t)-1);
+    return strncasecmp(s1, s2, HIGHEST_POSITIVE);
 }
 #endif
 
 /* This function is equivalent to strcasecmp() for multibyte strings. */
 int mbstrcasecmp(const char *s1, const char *s2)
 {
-    return mbstrncasecmp(s1, s2, (size_t)-1);
+    return mbstrncasecmp(s1, s2, HIGHEST_POSITIVE);
 }
 
 #ifndef HAVE_STRNCASECMP
@@ -530,53 +483,39 @@ int nstrncasecmp(const char *s1, const char *s2, size_t n)
 }
 #endif
 
-/* This function is equivalent to strncasecmp() for multibyte
- * strings. */
+/* This function is equivalent to strncasecmp() for multibyte strings. */
 int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 {
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	char *s1_mb, *s2_mb;
-	wchar_t ws1, ws2;
+	wchar_t wc1, wc2;
 
 	if (s1 == s2)
 	    return 0;
 
 	assert(s1 != NULL && s2 != NULL);
 
-	s1_mb = charalloc(MB_CUR_MAX);
-	s2_mb = charalloc(MB_CUR_MAX);
+	for (; *s1 != '\0' && *s2 != '\0' && n > 0;
+		s1 += move_mbright(s1, 0), s2 += move_mbright(s2, 0), n--) {
+	    bool bad1 = FALSE, bad2 = FALSE;
 
-	for (; *s1 != '\0' && *s2 != '\0' && n > 0; s1 +=
-		move_mbright(s1, 0), s2 += move_mbright(s2, 0), n--) {
-	    bool bad_s1_mb = FALSE, bad_s2_mb = FALSE;
-	    int s1_mb_len, s2_mb_len;
-
-	    s1_mb_len = parse_mbchar(s1, s1_mb, NULL);
-
-	    if (mbtowc(&ws1, s1_mb, s1_mb_len) < 0) {
+	    if (mbtowc(&wc1, s1, MB_CUR_MAX) < 0) {
 		mbtowc_reset();
-		ws1 = (unsigned char)*s1_mb;
-		bad_s1_mb = TRUE;
+		wc1 = (unsigned char)*s1;
+		bad1 = TRUE;
 	    }
 
-	    s2_mb_len = parse_mbchar(s2, s2_mb, NULL);
-
-	    if (mbtowc(&ws2, s2_mb, s2_mb_len) < 0) {
+	    if (mbtowc(&wc2, s2, MB_CUR_MAX) < 0) {
 		mbtowc_reset();
-		ws2 = (unsigned char)*s2_mb;
-		bad_s2_mb = TRUE;
+		wc2 = (unsigned char)*s2;
+		bad2 = TRUE;
 	    }
 
-	    if (bad_s1_mb != bad_s2_mb || towlower(ws1) !=
-		towlower(ws2))
+	    if (bad1 != bad2 || towlower(wc1) != towlower(wc2))
 		break;
 	}
 
-	free(s1_mb);
-	free(s2_mb);
-
-	return (n > 0) ? towlower(ws1) - towlower(ws2) : 0;
+	return (n > 0) ? towlower(wc1) - towlower(wc2) : 0;
     } else
 #endif
 	return strncasecmp(s1, s2, n);
@@ -623,7 +562,8 @@ char *mbstrcasestr(const char *haystack, const char *needle)
 
 	for (; *haystack != '\0' && haystack_len >= needle_len;
 		haystack += move_mbright(haystack, 0), haystack_len--) {
-	    if (mbstrncasecmp(haystack, needle, needle_len) == 0)
+	    if (mbstrncasecmp(haystack, needle, needle_len) == 0 &&
+			mblen(haystack, MB_CUR_MAX) > 0)
 		return (char *)haystack;
 	}
 
@@ -693,14 +633,12 @@ char *revstrcasestr(const char *haystack, const char *needle, const char
 }
 
 /* This function is equivalent to strcasestr() for multibyte strings,
- * except in that it scans the string in reverse, starting at
- * rev_start. */
+ * except in that it scans the string in reverse, starting at rev_start. */
 char *mbrevstrcasestr(const char *haystack, const char *needle, const
 	char *rev_start)
 {
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	bool begin_line = FALSE;
 	size_t rev_start_len, needle_len;
 
 	assert(haystack != NULL && needle != NULL && rev_start != NULL);
@@ -715,21 +653,19 @@ char *mbrevstrcasestr(const char *haystack, const char *needle, const
 
 	rev_start_len = mbstrlen(rev_start);
 
-	while (!begin_line) {
-	    if (rev_start_len >= needle_len && mbstrncasecmp(rev_start,
-		needle, needle_len) == 0)
+	while (TRUE) {
+	    if (rev_start_len >= needle_len &&
+			mbstrncasecmp(rev_start, needle, needle_len) == 0 &&
+			mblen(rev_start, MB_CUR_MAX) > 0)
 		return (char *)rev_start;
 
+	    /* If we've reached the head of the haystack, we found nothing. */
 	    if (rev_start == haystack)
-		begin_line = TRUE;
-	    else {
-		rev_start = haystack + move_mbleft(haystack, rev_start -
-			haystack);
-		rev_start_len++;
-	    }
-	}
+		return NULL;
 
-	return NULL;
+	    rev_start = haystack + move_mbleft(haystack, rev_start - haystack);
+	    rev_start_len++;
+	}
     } else
 #endif
 	return revstrcasestr(haystack, needle, rev_start);
@@ -788,9 +724,8 @@ char *mbstrchr(const char *s, const char *c)
 	char *s_mb = charalloc(MB_CUR_MAX);
 	const char *q = s;
 	wchar_t ws, wc;
-	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
 
-	if (c_mb_len < 0) {
+	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
 	    mbtowc_reset();
 	    wc = (unsigned char)*c;
 	    bad_c_mb = TRUE;
@@ -850,11 +785,14 @@ char *revstrpbrk(const char *s, const char *accept, const char
 {
     assert(s != NULL && accept != NULL && rev_start != NULL);
 
-    for (; rev_start >= s; rev_start--) {
-	const char *q = (*rev_start == '\0') ? NULL : strchr(accept,
-		*rev_start);
+    if (*rev_start == '\0') {
+	if (rev_start == s)
+	   return NULL;
+	rev_start--;
+    }
 
-	if (q != NULL)
+    for (; rev_start >= s; rev_start--) {
+	if (strchr(accept, *rev_start) != NULL)
 	    return (char *)rev_start;
     }
 
@@ -862,8 +800,7 @@ char *revstrpbrk(const char *s, const char *accept, const char
 }
 
 /* This function is equivalent to strpbrk() for multibyte strings,
- * except in that it scans the string in reverse, starting at
- * rev_start. */
+ * except in that it scans the string in reverse, starting at rev_start. */
 char *mbrevstrpbrk(const char *s, const char *accept, const char
 	*rev_start)
 {
@@ -871,29 +808,29 @@ char *mbrevstrpbrk(const char *s, const char *accept, const char
 
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	bool begin_line = FALSE;
-
-	while (!begin_line) {
-	    const char *q = (*rev_start == '\0') ? NULL :
-		mbstrchr(accept, rev_start);
-
-	    if (q != NULL)
-		return (char *)rev_start;
-
+	if (*rev_start == '\0') {
 	    if (rev_start == s)
-		begin_line = TRUE;
-	    else
-		rev_start = s + move_mbleft(s, rev_start - s);
+		return NULL;
+	    rev_start = s + move_mbleft(s, rev_start - s);
 	}
 
-	return NULL;
+	while (TRUE) {
+	    if (mbstrchr(accept, rev_start) != NULL)
+		return (char *)rev_start;
+
+	    /* If we've reached the head of the string, we found nothing. */
+	    if (rev_start == s)
+		return NULL;
+
+	    rev_start = s + move_mbleft(s, rev_start - s);
+	}
     } else
 #endif
 	return revstrpbrk(s, accept, rev_start);
 }
 #endif /* !NANO_TINY */
 
-#if defined(ENABLE_NANORC) && (!defined(NANO_TINY) || !defined(DISABLE_JUSTIFY))
+#if !defined(DISABLE_NANORC) && (!defined(NANO_TINY) || !defined(DISABLE_JUSTIFY))
 /* Return TRUE if the string s contains one or more blank characters,
  * and FALSE otherwise. */
 bool has_blank_chars(const char *s)
@@ -935,19 +872,20 @@ bool has_blank_mbchars(const char *s)
 #endif
 	return has_blank_chars(s);
 }
-#endif /* ENABLE_NANORC && (!NANO_TINY || !DISABLE_JUSTIFY) */
+#endif /* !DISABLE_NANORC && (!NANO_TINY || !DISABLE_JUSTIFY) */
 
 #ifdef ENABLE_UTF8
 /* Return TRUE if wc is valid Unicode, and FALSE otherwise. */
 bool is_valid_unicode(wchar_t wc)
 {
-    return ((0 <= wc && wc <= 0x10FFFF) && (wc <= 0xD7FF || 0xE000 <=
-	wc) && (wc <= 0xFDCF || 0xFDF0 <= wc) && ((wc & 0xFFFF) <=
-	0xFFFD));
+    return ((0 <= wc && wc <= 0xD7FF) ||
+		(0xE000 <= wc && wc <= 0xFDCF) ||
+		(0xFDF0 <= wc && wc <= 0xFFFD) ||
+		(0xFFFF < wc && wc <= 0x10FFFF && (wc & 0xFFFF) <= 0xFFFD));
 }
 #endif
 
-#ifdef ENABLE_NANORC
+#ifndef DISABLE_NANORC
 /* Check if the string s is a valid multibyte string.  Return TRUE if it
  * is, and FALSE otherwise. */
 bool is_valid_mbstring(const char *s)
@@ -960,4 +898,4 @@ bool is_valid_mbstring(const char *s)
 #endif
 	TRUE;
 }
-#endif /* ENABLE_NANORC */
+#endif /* !DISABLE_NANORC */
