@@ -21,6 +21,7 @@
  **************************************************************************/
 
 #include "proto.h"
+#include "revision.h"
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -218,17 +219,15 @@ partition *partition_filestruct(filestruct *top, size_t top_x,
     } else
 	p->filebot = NULL;
 
-    /* Save the line above the top of the partition, detach the top of
-     * the partition from it, and save the text before top_x in
-     * top_data. */
+    /* Remember which line is above the top of the partition, detach the
+     * top of the partition from it, and save the text before top_x. */
     p->top_prev = top->prev;
     top->prev = NULL;
     p->top_data = mallocstrncpy(NULL, top->data, top_x + 1);
     p->top_data[top_x] = '\0';
 
-    /* Save the line below the bottom of the partition, detach the
-     * bottom of the partition from it, and save the text after bot_x in
-     * bot_data. */
+    /* Remember which line is below the bottom of the partition, detach the
+     * bottom of the partition from it, and save the text after bot_x. */
     p->bot_next = bot->next;
     bot->next = NULL;
     p->bot_data = mallocstrcpy(NULL, bot->data + bot_x);
@@ -237,8 +236,7 @@ partition *partition_filestruct(filestruct *top, size_t top_x,
     null_at(&bot->data, bot_x);
 
     /* Remove all text before top_x at the top of the partition. */
-    charmove(top->data, top->data + top_x, strlen(top->data) -
-	top_x + 1);
+    charmove(top->data, top->data + top_x, strlen(top->data) - top_x + 1);
     align(&top->data);
 
     /* Return the partition. */
@@ -249,23 +247,20 @@ partition *partition_filestruct(filestruct *top, size_t top_x,
  * at (filebot, strlen(filebot->data)) again. */
 void unpartition_filestruct(partition **p)
 {
-    char *tmp;
-
     assert(p != NULL && openfile->fileage != NULL && openfile->filebot != NULL);
 
     /* Reattach the line above the top of the partition, and restore the
      * text before top_x from top_data.  Free top_data when we're done
      * with it. */
-    tmp = mallocstrcpy(NULL, openfile->fileage->data);
     openfile->fileage->prev = (*p)->top_prev;
     if (openfile->fileage->prev != NULL)
 	openfile->fileage->prev->next = openfile->fileage;
     openfile->fileage->data = charealloc(openfile->fileage->data,
-	strlen((*p)->top_data) + strlen(openfile->fileage->data) + 1);
-    strcpy(openfile->fileage->data, (*p)->top_data);
+		strlen((*p)->top_data) + strlen(openfile->fileage->data) + 1);
+    charmove(openfile->fileage->data + strlen((*p)->top_data),
+		openfile->fileage->data, strlen(openfile->fileage->data) + 1);
+    strncpy(openfile->fileage->data, (*p)->top_data, strlen((*p)->top_data));
     free((*p)->top_data);
-    strcat(openfile->fileage->data, tmp);
-    free(tmp);
 
     /* Reattach the line below the bottom of the partition, and restore
      * the text after bot_x from bot_data.  Free bot_data when we're
@@ -274,7 +269,7 @@ void unpartition_filestruct(partition **p)
     if (openfile->filebot->next != NULL)
 	openfile->filebot->next->prev = openfile->filebot;
     openfile->filebot->data = charealloc(openfile->filebot->data,
-	strlen(openfile->filebot->data) + strlen((*p)->bot_data) + 1);
+		strlen(openfile->filebot->data) + strlen((*p)->bot_data) + 1);
     strcat(openfile->filebot->data, (*p)->bot_data);
     free((*p)->bot_data);
 
@@ -315,9 +310,8 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
      * the edit window is inside the partition, and keep track of
      * whether the mark begins inside the partition. */
     filepart = partition_filestruct(top, top_x, bot, bot_x);
-    edittop_inside = (openfile->edittop->lineno >=
-	openfile->fileage->lineno && openfile->edittop->lineno <=
-	openfile->filebot->lineno);
+    edittop_inside = (openfile->edittop->lineno >= openfile->fileage->lineno &&
+			openfile->edittop->lineno <= openfile->filebot->lineno);
 #ifndef NANO_TINY
     if (openfile->mark_set) {
 	mark_inside = (openfile->mark_begin->lineno >=
@@ -332,8 +326,7 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
     }
 #endif
 
-    /* Get the number of characters in the text, and subtract it from
-     * totsize. */
+    /* Subtract the number of characters in the text from the file size. */
     openfile->totsize -= get_totsize(top, bot);
 
     if (*file_top == NULL) {
@@ -343,7 +336,7 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
 	*file_top = openfile->fileage;
 	*file_bot = openfile->filebot;
 
-	/* Renumber starting with file_top. */
+	/* Renumber, starting with file_top. */
 	renumber(*file_top);
     } else {
 	filestruct *file_bot_save = *file_bot;
@@ -366,13 +359,11 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
 
 	delete_node(openfile->fileage);
 
-	/* Renumber starting with the line after the original
-	 * file_bot. */
+	/* Renumber, starting with the line after the original file_bot. */
 	renumber(file_bot_save->next);
     }
 
-    /* Since the text has now been saved, remove it from the
-     * filestruct. */
+    /* Since the text has now been saved, remove it from the filestruct. */
     openfile->fileage = (filestruct *)nmalloc(sizeof(filestruct));
     openfile->fileage->data = mallocstrcpy(NULL, "");
     openfile->filebot = openfile->fileage;
@@ -408,12 +399,10 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
 	refresh_needed = TRUE;
     }
 
-    /* Renumber starting with the beginning line of the old
-     * partition. */
+    /* Renumber, starting with the beginning line of the old partition. */
     renumber(top_save);
 
-    /* If the NO_NEWLINES flag isn't set, and the text doesn't end with
-     * a magicline, add a new magicline. */
+    /* If the text doesn't end with a magicline, and it should, add one. */
     if (!ISSET(NO_NEWLINES) && openfile->filebot->data[0] != '\0')
 	new_magicline();
 }
@@ -439,22 +428,21 @@ void copy_from_filestruct(filestruct *somebuffer)
 	size_t top_x, bot_x;
 
 	mark_order((const filestruct **)&top, &top_x,
-		(const filestruct **)&bot, &bot_x, &right_side_up);
+			(const filestruct **)&bot, &bot_x, &right_side_up);
 
 	single_line = (top == bot);
     }
 #endif
 
-    /* Partition the filestruct so that it contains no text, and keep
-     * track of whether the top of the edit window is inside the
-     * partition. */
-    filepart = partition_filestruct(openfile->current,
-	openfile->current_x, openfile->current, openfile->current_x);
+    /* Partition the filestruct so that it contains no text, and remember
+     * whether the current line is at the top of the edit window. */
+    filepart = partition_filestruct(openfile->current, openfile->current_x,
+				openfile->current, openfile->current_x);
     edittop_inside = (openfile->edittop == openfile->fileage);
+    free_filestruct(openfile->fileage);
 
     /* Put the top and bottom of the current filestruct at the top and
      * bottom of a copy of the passed buffer. */
-    free_filestruct(openfile->fileage);
     openfile->fileage = copy_filestruct(somebuffer);
     openfile->filebot = openfile->fileage;
     while (openfile->filebot->next != NULL)
@@ -489,33 +477,29 @@ void copy_from_filestruct(filestruct *somebuffer)
     }
 #endif
 
-    /* Get the number of characters in the copied text, and add it to
-     * totsize. */
-    openfile->totsize += get_totsize(openfile->fileage,
-	openfile->filebot);
+    /* Add the number of characters in the copied text to the file size. */
+    openfile->totsize += get_totsize(openfile->fileage, openfile->filebot);
 
     /* Update the current y-coordinate to account for the number of
      * lines the copied text has, less one since the first line will be
      * tacked onto the current line. */
     openfile->current_y += openfile->filebot->lineno - 1;
 
-    top_save = openfile->fileage;
-
-    /* If the top of the edit window is inside the partition, set it to
-     * where the copied text now starts. */
+    /* If we pasted onto the first line of the edit window, the corresponding
+     * struct has been freed, so... point at the start of the copied text. */
     if (edittop_inside)
 	openfile->edittop = openfile->fileage;
 
+    top_save = openfile->fileage;
+
     /* Unpartition the filestruct so that it contains all the text
      * again, plus the copied text. */
     unpartition_filestruct(&filepart);
 
-    /* Renumber starting with the beginning line of the old
-     * partition. */
+    /* Renumber, starting with the beginning line of the old partition. */
     renumber(top_save);
 
-    /* If the NO_NEWLINES flag isn't set, and the text doesn't end with
-     * a magicline, add a new magicline. */
+    /* If the text doesn't end with a magicline, and it should, add one. */
     if (!ISSET(NO_NEWLINES) && openfile->filebot->data[0] != '\0')
 	new_magicline();
 }
@@ -705,11 +689,14 @@ void die_save_file(const char *die_filename
     free(targetname);
 }
 
+#define TOP_ROWS    (ISSET(MORE_SPACE) ? 1 : 2)
+#define BOTTOM_ROWS    (ISSET(NO_HELP) ? 1 : 3)
+
 /* Initialize the three window portions nano uses. */
 void window_init(void)
 {
     /* If the screen height is too small, get out. */
-    editwinrows = LINES - 5 + more_space() + no_help();
+    editwinrows = LINES - TOP_ROWS - BOTTOM_ROWS;
     if (COLS < MIN_EDITOR_COLS || editwinrows < MIN_EDITOR_ROWS)
 	die(_("Window size is too small for nano...\n"));
 
@@ -730,10 +717,9 @@ void window_init(void)
 	delwin(bottomwin);
 
     /* Set up the windows. */
-    topwin = newwin(2 - more_space(), COLS, 0, 0);
-    edit = newwin(editwinrows, COLS, 2 - more_space(), 0);
-    bottomwin = newwin(3 - no_help(), COLS, editwinrows + (2 -
-	more_space()), 0);
+    topwin = newwin(TOP_ROWS, COLS, 0, 0);
+    edit = newwin(editwinrows, COLS, TOP_ROWS, 0);
+    bottomwin = newwin(BOTTOM_ROWS, COLS, TOP_ROWS + editwinrows, 0);
 
     /* Turn the keypad on for the windows, if necessary. */
     if (!ISSET(REBIND_KEYPAD)) {
@@ -876,6 +862,8 @@ void usage(void)
 #ifndef NANO_TINY
     print_opt("-W", "--wordbounds",
 	N_("Detect word boundaries more accurately"));
+    print_opt("-X", "--wordchars",
+	N_("Which other characters are word parts"));
 #endif
 #ifndef DISABLE_COLOR
     if (!ISSET(RESTRICTED))
@@ -935,14 +923,14 @@ void usage(void)
 void version(void)
 {
 #ifdef REVISION
-    printf(" nano from git, commit %s (after %s)\n", REVISION, VERSION);
+    printf(" nano from git, %s\n", REVISION);
 #else
     printf(_(" nano, version %s\n"), VERSION);
 #endif
     printf(" (C) 1999..2016 Free Software Foundation, Inc.\n");
     printf(_(" (C) 2014..%s the contributors to nano\n"), "2016");
     printf(
-	_(" Email: nano@nano-editor.org	Web: http://www.nano-editor.org/"));
+	_(" Email: nano@nano-editor.org	Web: https://nano-editor.org/"));
     printf(_("\n Compiled options:"));
 
 #ifdef NANO_TINY
@@ -1057,21 +1045,6 @@ void version(void)
     printf("\n");
 }
 
-/* Return 1 if the MORE_SPACE flag is set, and 0 otherwise.  This is
- * used to calculate the sizes and Y coordinates of the subwindows. */
-int more_space(void)
-{
-    return ISSET(MORE_SPACE) ? 1 : 0;
-}
-
-/* Return 2 if the NO_HELP flag is set, and 0 otherwise.  This is used
- * to calculate the sizes and Y coordinates of the subwindows, because
- * having NO_HELP adds two lines to the edit window. */
-int no_help(void)
-{
-    return ISSET(NO_HELP) ? 2 : 0;
-}
-
 /* Indicate that the current file has no name, in a way that gets the
  * user's attention.  This is used when trying to save a file with no
  * name with the TEMP_FILE flag set, just before the filename prompt. */
@@ -1126,8 +1099,6 @@ void do_exit(void)
 	close_and_go();
     else if (i != 1)
 	statusbar(_("Cancelled"));
-
-    display_main_list();
 }
 
 /* Close the current buffer, and terminate nano if it was the last. */
@@ -1316,10 +1287,9 @@ RETSIGTYPE do_continue(int signal)
 #endif
 
 #ifndef NANO_TINY
-    /* Perhaps the user resized the window while we slept.  Handle it,
-     * and restore the terminal to its previous state and update the
-     * screen in the process. */
-    handle_sigwinch(0);
+    /* Perhaps the user resized the window while we slept.  So act as if,
+     * and restore the terminal to its previous state in the process. */
+    regenerate_screen();
 #else
     /* Restore the terminal to its previous state. */
     terminal_init();
@@ -1572,33 +1542,33 @@ void unbound_key(int code)
     if (func_key)
 	statusline(ALERT, _("Unbound key"));
     else if (meta_key) {
-	if (0x60 < code && code < 0x7B)
-	    code -= 0x20;
-	statusline(ALERT, _("Unbound key: M-%c"), code);
+	if (code == '[')
+	    statusline(ALERT, _("Unbindable key: M-["));
+	else
+	    statusline(ALERT, _("Unbound key: M-%c"), toupper(code));
     } else if (code < 0x20)
 	statusline(ALERT, _("Unbound key: ^%c"), code + 0x40);
     else
 	statusline(ALERT, _("Unbound key: %c"), code);
 }
 
-/* Read in a character, interpret it as a shortcut or toggle if
- * necessary, and return it.
- * If allow_funcs is FALSE, don't actually run any functions associated
- * with shortcut keys. */
+/* Read in a keystroke.  Act on the keystroke if it is a shortcut or a toggle;
+ * otherwise, insert it into the edit buffer.  If allow_funcs is FALSE, don't
+ * do anything with the keystroke -- just return it. */
 int do_input(bool allow_funcs)
 {
     int input;
-	/* The character we read in. */
-    static int *kbinput = NULL;
-	/* The input buffer. */
-    static size_t kbinput_len = 0;
+	/* The keystroke we read in: a character or a shortcut. */
+    static char *puddle = NULL;
+	/* The input buffer for actual characters. */
+    static size_t depth = 0;
 	/* The length of the input buffer. */
     bool preserve = FALSE;
-	/* Preserve the contents of the cutbuffer? */
+	/* Whether to preserve the contents of the cutbuffer. */
     const sc *s;
     bool have_shortcut;
 
-    /* Read in a character. */
+    /* Read in a keystroke. */
     input = get_kbinput(edit);
 
 #ifndef NANO_TINY
@@ -1638,26 +1608,23 @@ int do_input(bool allow_funcs)
     }
 
     if (allow_funcs) {
-	/* If we got a character, and it isn't a shortcut or toggle,
-	 * it's a normal text character.  Display the warning if we're
-	 * in view mode, or add the character to the input buffer if
-	 * we're not. */
+	/* If the keystroke isn't a shortcut nor a toggle, it's a normal text
+	 * character: add the character to the input buffer -- or display a
+	 * warning when we're in view mode. */
 	if (input != ERR && !have_shortcut) {
 	    if (ISSET(VIEW_MODE))
 		print_view_warning();
 	    else {
-		kbinput_len++;
-		kbinput = (int *)nrealloc(kbinput, kbinput_len *
-			sizeof(int));
-		kbinput[kbinput_len - 1] = input;
+		/* Store the byte, and leave room for a terminating zero. */
+		puddle = charealloc(puddle, depth + 2);
+		puddle[depth++] = (char)input;
 	    }
 	}
 
 	/* If we got a shortcut or toggle, or if there aren't any other
-	 * characters waiting after the one we read in, we need to
-	 * output all the characters in the input buffer if it isn't
-	 * empty.  Note that it should be empty if we're in view
-	 * mode. */
+	 * characters waiting after the one we read in, we need to output
+	 * all available characters in the input puddle.  Note that this
+	 * puddle will be empty if we're in view mode. */
 	if (have_shortcut || get_key_buffer_len() == 0) {
 #ifndef DISABLE_WRAPPING
 	    /* If we got a shortcut or toggle, and it's not the shortcut
@@ -1666,24 +1633,16 @@ int do_input(bool allow_funcs)
 		wrap_reset();
 #endif
 
-	    if (kbinput != NULL) {
-		/* Display all the characters in the input buffer at
-		 * once, filtering out control characters. */
-		char *output = charalloc(kbinput_len + 1);
-		size_t i;
-
-		for (i = 0; i < kbinput_len; i++)
-		    output[i] = (char)kbinput[i];
-		output[i] = '\0';
-
-		do_output(output, kbinput_len, FALSE);
-
-		free(output);
+	    if (puddle != NULL) {
+		/* Insert all bytes in the input buffer into the edit buffer
+		 * at once, filtering out any low control codes. */
+		puddle[depth] = '\0';
+		do_output(puddle, depth, FALSE);
 
 		/* Empty the input buffer. */
-		kbinput_len = 0;
-		free(kbinput);
-		kbinput = NULL;
+		free(puddle);
+		puddle = NULL;
+		depth = 0;
 	    }
 	}
 
@@ -1864,12 +1823,11 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 #endif
 
     while (i < output_len) {
-	/* If allow_cntrls is TRUE, convert nulls and newlines properly. */
+	/* If control codes are allowed, encode a verbatim null as a newline,
+	 * and let a verbatim ^J create a whole new line. */
 	if (allow_cntrls) {
-	    /* Null to newline, if needed. */
 	    if (output[i] == '\0')
 		output[i] = '\n';
-	    /* Newline to Enter, if needed. */
 	    else if (output[i] == '\n') {
 		do_enter();
 		i++;
@@ -1877,31 +1835,29 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 	    }
 	}
 
-	/* Interpret the next multibyte character. */
+	/* Get the next multibyte character. */
 	char_buf_len = parse_mbchar(output + i, char_buf, NULL);
 
 	i += char_buf_len;
 
-	/* If allow_cntrls is FALSE, filter out an ASCII control character. */
+	/* If controls are not allowed, ignore an ASCII control character. */
 	if (!allow_cntrls && is_ascii_cntrl_char(*(output + i -	char_buf_len)))
 	    continue;
 
-	/* If the NO_NEWLINES flag isn't set, when a character is
-	 * added to the magicline, it means we need a new magicline. */
+	/* If we're adding to the magicline, create a new magicline. */
 	if (!ISSET(NO_NEWLINES) && openfile->filebot == openfile->current)
 	    new_magicline();
 
-	/* More dangerousness fun =) */
-	openfile->current->data = charealloc(openfile->current->data,
-					current_len + char_buf_len + 1);
-
 	assert(openfile->current_x <= current_len);
 
+	/* Make room for the new character and copy it into the line. */
+	openfile->current->data = charealloc(openfile->current->data,
+					current_len + char_buf_len + 1);
 	charmove(openfile->current->data + openfile->current_x + char_buf_len,
 			openfile->current->data + openfile->current_x,
 			current_len - openfile->current_x + 1);
 	strncpy(openfile->current->data + openfile->current_x, char_buf,
-		char_buf_len);
+			char_buf_len);
 	current_len += char_buf_len;
 	openfile->totsize++;
 	set_modified();
@@ -2029,6 +1985,7 @@ int main(int argc, char **argv)
 	{"smooth", 0, NULL, 'S'},
 	{"quickblank", 0, NULL, 'U'},
 	{"wordbounds", 0, NULL, 'W'},
+	{"wordchars", 1, NULL, 'X'},
 	{"autoindent", 0, NULL, 'i'},
 	{"cut", 0, NULL, 'k'},
 	{"unix", 0, NULL, 'u'},
@@ -2074,11 +2031,11 @@ int main(int argc, char **argv)
     while ((optchr =
 #ifdef HAVE_GETOPT_LONG
 	getopt_long(argc, argv,
-		"ABC:DEFGHIKLNOPQ:RST:UVWY:abcdefghijklmno:pqr:s:tuvwxz$",
+		"ABC:DEFGHIKLNOPQ:RST:UVWX:Y:abcdefghijklmno:pqr:s:tuvwxz$",
 		long_options, NULL)
 #else
 	getopt(argc, argv,
-		"ABC:DEFGHIKLNOPQ:RST:UVWY:abcdefghijklmno:pqr:s:tuvwxz$")
+		"ABC:DEFGHIKLNOPQ:RST:UVWX:Y:abcdefghijklmno:pqr:s:tuvwxz$")
 #endif
 		) != -1) {
 	switch (optchr) {
@@ -2180,6 +2137,9 @@ int main(int argc, char **argv)
 	    case 'W':
 		SET(WORD_BOUNDS);
 		break;
+	    case 'X':
+		word_chars = mallocstrcpy(word_chars, optarg);
+		break;
 #endif
 #ifndef DISABLE_COLOR
 	    case 'Y':
@@ -2313,6 +2273,7 @@ int main(int argc, char **argv)
 #endif
 #ifndef NANO_TINY
 	char *backup_dir_cpy = backup_dir;
+	char *word_chars_cpy = word_chars;
 #endif
 #ifndef DISABLE_JUSTIFY
 	char *quotestr_cpy = quotestr;
@@ -2331,6 +2292,7 @@ int main(int argc, char **argv)
 #endif
 #ifndef NANO_TINY
 	backup_dir = NULL;
+	word_chars = NULL;
 #endif
 #ifndef DISABLE_JUSTIFY
 	quotestr = NULL;
@@ -2361,6 +2323,10 @@ int main(int argc, char **argv)
 	    free(backup_dir);
 	    backup_dir = backup_dir_cpy;
 	}
+	if (word_chars_cpy != NULL) {
+	    free(word_chars);
+	    word_chars = word_chars_cpy;
+	}
 #endif
 #ifndef DISABLE_JUSTIFY
 	if (quotestr_cpy != NULL) {
@@ -2540,25 +2506,32 @@ int main(int argc, char **argv)
 #ifndef DISABLE_COLOR
     set_colorpairs();
 #else
-    interface_color_pair[TITLE_BAR].pairnum = hilite_attribute;
-    interface_color_pair[STATUS_BAR].pairnum = hilite_attribute;
-    interface_color_pair[KEY_COMBO].pairnum = hilite_attribute;
-    interface_color_pair[FUNCTION_TAG].pairnum = A_NORMAL;
-    interface_color_pair[TITLE_BAR].bright = FALSE;
-    interface_color_pair[STATUS_BAR].bright = FALSE;
-    interface_color_pair[KEY_COMBO].bright = FALSE;
-    interface_color_pair[FUNCTION_TAG].bright = FALSE;
+    interface_color_pair[TITLE_BAR] = hilite_attribute;
+    interface_color_pair[STATUS_BAR] = hilite_attribute;
+    interface_color_pair[KEY_COMBO] = hilite_attribute;
+    interface_color_pair[FUNCTION_TAG] = A_NORMAL;
 #endif
 
 #if !defined(NANO_TINY) && defined(HAVE_KEY_DEFINED)
     const char *keyvalue;
-    /* Ask ncurses for the key codes for Control+Left and Control+Right. */
+    /* Ask ncurses for the key codes for Control+Left/Right/Up/Down. */
     keyvalue = tigetstr("kLFT5");
     if (keyvalue != 0 && keyvalue != (char *)-1)
 	controlleft = key_defined(keyvalue);
     keyvalue = tigetstr("kRIT5");
     if (keyvalue != 0 && keyvalue != (char *)-1)
 	controlright = key_defined(keyvalue);
+    keyvalue = tigetstr("kUP5");
+    if (keyvalue != 0 && keyvalue != (char *)-1)
+	controlup = key_defined(keyvalue);
+    keyvalue = tigetstr("kDN5");
+    if (keyvalue != 0 && keyvalue != (char *)-1)
+	controldown = key_defined(keyvalue);
+#endif
+
+#ifndef USE_SLANG
+    /* Tell ncurses to pass the Esc key quickly. */
+    set_escdelay(50);
 #endif
 
 #ifdef DEBUG
@@ -2652,12 +2625,12 @@ int main(int argc, char **argv)
     fprintf(stderr, "Main: bottom win, top win and edit win\n");
 #endif
 
-    display_main_list();
-
     display_buffer();
 
     while (TRUE) {
-	currmenu = MMAIN;
+	if (currmenu != MMAIN)
+	    display_main_list();
+
 	focusing = TRUE;
 	lastmessage = HUSH;
 
@@ -2675,7 +2648,7 @@ int main(int argc, char **argv)
 	curs_set(1);
 	wnoutrefresh(edit);
 
-	/* Read in and interpret characters. */
+	/* Read in and interpret keystrokes. */
 	do_input(TRUE);
     }
 
